import { watch } from 'chokidar';
import glob from 'glob';
import { spawn } from 'node:child_process';
import { mkdir, readFile, writeFile } from 'node:fs/promises';
import { resolve } from 'node:path';
import { Worker } from 'node:worker_threads';
import { finalizePage } from './css.js';
import { rootDir, swc } from './models.js';
export function developmentBuilder(logger) {
    let compiling = false;
    let success;
    let fail;
    const promise = new Promise((resolve, reject)=>{
        success = resolve;
        fail = reject;
    });
    let timeout;
    function scheduleRun() {
        if (timeout) {
            clearTimeout(timeout);
        }
        timeout = setTimeout(run, 10);
    }
    function run() {
        timeout = null;
        if (compiling) {
            return;
        }
        compiling = true;
        const compilation = spawn(swc, [
            '-d',
            'tmp',
            'src'
        ]);
        let error = Buffer.alloc(0);
        compilation.stderr.on('data', (chunk)=>{
            error = Buffer.concat([
                error,
                chunk
            ]);
        });
        compilation.on('close', (code)=>{
            if (code !== 0) {
                compiling = false;
                const errorString = error.toString().trim().replaceAll(/(^.)/gm, '$1'.padStart(17, ' ')).replaceAll(rootDir, '$ROOT');
                logger.error('Code compilation failed:\n\n  ' + errorString + '\n');
                return;
            }
            const worker = new Worker(new URL('buildWorker.js', import.meta.url), {
                workerData: {
                    mode: 'development'
                }
            });
            worker.on('error', (error)=>{
                compiling = false;
                fail(error);
            });
            worker.on('exit', ()=>{
                compiling = false;
            });
        });
    }
    const watcher = watch([
        resolve(rootDir, 'src'),
        resolve(rootDir, 'config')
    ], {
        persistent: true
    });
    process.on('SIGINT', ()=>{
        watcher.close().then(success).catch(fail);
    });
    watcher.on('add', scheduleRun).on('change', scheduleRun).on('unlink', scheduleRun).on('error', fail);
    return promise;
}
export async function productionBuilder() {
    // First of all create the site
    await mkdir(resolve(rootDir, 'dist'), {
        recursive: true
    });
    const { build , createStylesheet , safelist  } = await import(resolve(rootDir, 'tmp/build/index.js'));
    await build('production', async (context)=>{
        const stylesheet = await createStylesheet(context.cssClasses, true);
        // Now, for each generated page, replace the @import class with the production CSS
        const pages = glob.sync(resolve(rootDir, 'dist', '**/*.html'));
        for (const page of pages){
            await writeFile(page, await finalizePage(await readFile(page, 'utf8'), stylesheet, safelist), 'utf8');
        }
    });
}
