import { createGenerator } from '@unocss/core';
import { transformDirectives } from '@unocss/transformer-directives';
import MagicString from 'magic-string';
import postcss from 'postcss';
import postcssDiscardComments from 'postcss-discard-comments';
import postcssImport from 'postcss-import';
import postcssMinifySelector from 'postcss-minify-selectors';
import postcssNested from 'postcss-nested';
import postcssNormalizeWhitespace from 'postcss-normalize-whitespace';
import { PurgeCSS } from 'purgecss';
import { rehype } from 'rehype';
import stringify from 'rehype-stringify';
import { visit } from 'unist-util-visit';
const cssClassAlphabet = 'abcdefghijklmnopqrstuvwxyz';
const cssClassAlphabetLength = cssClassAlphabet.length;
const cssForbiddenClasses = new Set([
    'ad'
]);
function cssClass(context) {
    do {
        let i = ++context.counter;
        let str = '';
        do {
            let index = i % cssClassAlphabetLength;
            i = i / cssClassAlphabetLength;
            if (index - 1 === -1) {
                index = cssClassAlphabetLength;
                i--;
            }
            str = cssClassAlphabet.charAt(index - 1) + str;
        }while (i >= 1)
        context.name = str;
    // Avoid some combinations
    }while (cssForbiddenClasses.has(context.name))
}
function extractCSSClassesPlugin() {
    return (tree, file)=>{
        const classes = new Set();
        const compressedClasses = new Map();
        file.data.classes = classes;
        file.data.compression = compressedClasses;
        visit(tree, 'element', (node)=>{
            var _node_properties;
            for (const klass of ((_node_properties = node.properties) === null || _node_properties === void 0 ? void 0 : _node_properties.className) ?? []){
                classes.add(klass);
            }
        });
    };
}
function compressCSSClassesPlugin(options = {}) {
    const safelist = new Set(options.safelist ?? []);
    return (tree, file)=>{
        const classGenerationContext = {
            name: '',
            counter: 0
        };
        const classes = new Set();
        const compressedClasses = new Map();
        file.data.classes = classes;
        file.data.compression = compressedClasses;
        visit(tree, 'element', (node)=>{
            var _node_properties;
            const klasses = [];
            for (const klass of ((_node_properties = node.properties) === null || _node_properties === void 0 ? void 0 : _node_properties.className) ?? []){
                if (safelist.has(klass)) {
                    // Do not compress safelist classes
                    klasses.push(klass);
                } else {
                    // Generate a new compressed class
                    if (!compressedClasses.has(klass)) {
                        cssClass(classGenerationContext);
                        compressedClasses.set(klass, classGenerationContext.name);
                    }
                    // Replace the class
                    klasses.push(compressedClasses.get(klass));
                }
            }
            if (klasses.length) {
                node.properties.className = klasses;
            }
        });
    };
}
export async function transformCSS(config, css) {
    // Create a generator
    const generator = createGenerator(config);
    // Trasform the file
    const code = new MagicString(css);
    await transformDirectives(code, generator, {});
    return code.toString();
}
async function finalizeCSS(css, minify, cssImporter) {
    const processed = await postcss([
        postcssImport({
            resolve (id) {
                return `/handled/${id}`;
            },
            async load (id) {
                id = id.replace(/^\/handled\//, '');
                const imported = await cssImporter(id);
                return imported ?? `/*!!! not-found: ${id} */`;
            }
        }),
        postcssNested(),
        ...minify ? [
            postcssDiscardComments({
                removeAll: true
            }),
            postcssNormalizeWhitespace(),
            postcssMinifySelector()
        ] : []
    ]).process(css, {
        from: 'input.css',
        to: 'output.css'
    });
    return processed.css;
}
export async function purgeCss(html, css) {
    const result = await new PurgeCSS().purge({
        content: [
            {
                raw: html,
                extension: 'html'
            }
        ],
        css: [
            {
                raw: css
            }
        ],
        safelist: [
            /\$|\[|:|(\\.)/
        ],
        variables: false
    });
    return result[0].css;
}
export function createBuildContext(isProduction, safelist) {
    return {
        isProduction,
        cssClasses: new Set(),
        safelist
    };
}
export async function prepareStyles(context, contents) {
    // First of all, extract classes and put them into the big list, which will only be processed once
    const parsedContents = await rehype().use(extractCSSClassesPlugin).process(contents);
    const allClasses = parsedContents.data.classes;
    for (const klass of allClasses){
        context.cssClasses.add(klass);
    }
    return '@import "/style.css";';
}
export async function createStylesheet(cssConfig, classes, minify, cssImporter, leadingCss = '') {
    // Generate the CSS out of the classes
    const generator = createGenerator({
        ...cssConfig,
        mergeSelectors: false
    });
    const commonCss = await transformCSS(cssConfig, leadingCss);
    const { css: generatedCss  } = await generator.generate(classes);
    return finalizeCSS(commonCss + generatedCss, minify, cssImporter);
}
// Compress each CSS class in the contents, then purged the CSS and finally replace the placeholder
export async function finalizePage(contents, stylesheet, safelist = []) {
    const css = await purgeCss(contents, stylesheet);
    // First of all, replace all classes with their compressed version
    const compressedContents = await rehype().use(compressCSSClassesPlugin, {
        safelist
    })// @ts-expect-error
    .use(stringify).process(contents);
    const compressedMap = compressedContents.data.compression;
    // Now grab the CSS and replace the same classes again
    const compressedCss = await postcss([
        {
            postcssPlugin: 'dante.compressor',
            Rule (decl) {
                if (!decl.selector.startsWith('.')) {
                    return;
                }
                const [klass, modifier] = decl.selector.slice(1).split(/(:(hover|focus|visited|active|link))/);
                if (safelist.includes(klass)) {
                    return;
                }
                const replacement = compressedMap.get(klass.replaceAll('\\', ''));
                if (replacement) {
                    const newSelector = [
                        '.',
                        replacement,
                        modifier
                    ].filter(Boolean).join('');
                    decl.selector = newSelector;
                } else {
                    decl.remove();
                }
            }
        }
    ]).process(css, {
        from: 'input.css',
        to: 'output.css'
    });
    return compressedContents.toString().replace('<style data-dante-placeholder="style">@import "/style.css";</style>', `<style>${compressedCss.css}</style>`);
}
